identification division.
program-id.    regexp.
|-----------------------------------------------------------------------------|
| Copyright (c) 1989-2010 by Micro Focus.   			              |
| Users of ACUCOBOL-GT may freely use this file.			      |
|-----------------------------------------------------------------------------|
|
|-----------------------------------------------------------------------------|
| This sample program illustrates the use of C$REGEXP to scan strings for     |
| patterns.  This is a very simple example, but does show all the uses of     |
| the new library routine.                                                    |
|-----------------------------------------------------------------------------|
environment division.
input-output section.
file-control.
    select search-file
    assign to input search-filename
    organization is line sequential
    file status is search-status.

data division.
file section.
*  The size of the record area of this file determines tha maximum
*  line length of the file we are searching.
fd  search-file.
01  search-record                   pic x(79).

working-storage section.
copy "def/acucobol.def".

01  search-status                   pic xx.
01  line-num                        pic 999.
01  col-num                         pic 999.
01  regular-expression-data.
    05  search-filename             pic x(80).
    05  regular-expression          pic x(80).
    05  filler                      pic x comp-x value 0.
    05  regexp-handle               handle.
    05  match-handle                handle.
    05  numgroups                   pic 999.
    05  idx                         pic 999.
    05  match-start                 pic 999.
    05  match-end                   pic 999.

procedure division.
main-paragraph.
    display window erase.
    call "C$REGEXP" using AREGEXP-GET-LEVEL.
    if return-code = AREGEXP-NONE
        display "This runtime doesn't support regular expressions"
        accept omitted
        stop run
    end-if.

    display "Filename" line 5 col 2.
    display "Regular expression" line 7 col 2.
    accept search-filename line 5 col 12.
*  The next statement will cause the COBOL program to abort if
*  the entered filename does not exist or has some other problem.
    open input search-file.
    accept regular-expression line 7 col 22.
*  For this sample, trailing spaces aren't allowed.
    inspect regular-expression replacing trailing spaces by low-values.
    call "C$REGEXP" using AREGEXP-COMPILE, regular-expression.
    move return-code to regexp-handle.
    if regexp-handle = null
        perform regexp-error
    end-if.
    move 8 to line-num.
    move 1 to col-num.
    perform until search-status = "10"
        read search-file next
          at end
            continue
          not at end
            call "C$REGEXP" using AREGEXP-MATCH, regexp-handle, search-record,
                    0, match-start, match-end
            move return-code to match-handle
            if match-start > 0
                display search-record line line-num no advancing
                perform until match-start = 0
                    display search-record(col-num + match-start - 1:
                                          match-end - match-start)
                            line line-num col col-num + match-start - 1
                            reverse no advancing
                    add match-end to col-num
                    call "C$REGEXP" using AREGEXP-MATCH, regexp-handle,
                            search-record(col-num:), 0,
                            match-start, match-end
                end-perform
                add 1 to line-num
                move 1 to col-num
                call "C$REGEXP" using AREGEXP-RELEASE-MATCH, match-handle
                if line-num > 24
                    display "Press return" line line-num
                    accept omitted
                    display window erase
                    move 1 to line-num
                end-if
            end-if
        end-read
    end-perform.

*  Clean up the resources we've used.
    accept omitted line line-num.
    call "C$REGEXP" using AREGEXP-RELEASE, regexp-handle.
    close search-file.
    stop run.

regexp-error.
    call "C$REGEXP" using AREGEXP-LAST-ERROR.
    display "Error: " line 24 return-code with convert.
    accept omitted.
    close search-file.
    stop run.
